'''
The code is based on
SimplerNetV1 in Pytorch.
https://github.com/Coderx7/SimpleNet_Pytorch
'''


import torch
import torch.nn as nn
from torch.autograd import Variable
import torch.nn.functional as F


class Cnn(nn.Module):
    def __init__(self, channels=1, classes=10, simpnet_name='cnn'):
        super(Cnn, self).__init__()
        self.features = self._make_layers(channels)
        self.classifier = nn.Sequential(
            nn.Linear(512, 1024),
            nn.ReLU(inplace=True),
            nn.Linear(1024, classes),
            )

    def load_my_state_dict(self, state_dict):

        own_state = self.state_dict()

        for name, param in state_dict.items():
            name = name.replace('module.', '')
            if name not in own_state:
                # print(name)
                continue
            if isinstance(param, Parameter):
                # backwards compatibility for serialized parameters
                param = param.data
            print("STATE_DICT: {}".format(name))
            try:
                own_state[name].copy_(param)
            except:
                print('While copying the parameter named {}, whose dimensions in the model are'
                      ' {} and whose dimensions in the checkpoint are {}, ... Using Initial Params'.format(
                    name, own_state[name].size(), param.size()))

    def forward(self, x):
        out = self.features(x)

        #Global Max Pooling
        out = F.max_pool2d(out, kernel_size=out.size()[2:]) 
        
        out = out.view(out.size(0), -1)
        out = self.classifier(out)
        return out

    def _make_layers(self, channels):

        model = nn.Sequential(
            nn.Conv2d(channels, 32, kernel_size=[5, 5], stride=(1, 1), padding=(1, 1)),
            nn.ReLU(inplace=True),
            nn.Conv2d(32, 64, kernel_size=[5, 5], stride=(1, 1), padding=(1, 1)),
            nn.ReLU(inplace=True),
            nn.MaxPool2d(kernel_size=(2, 2), stride=(2, 2), dilation=(1, 1), ceil_mode=False),

            nn.Conv2d(64, 512, kernel_size=[5, 5], stride=(1, 1), padding=(1, 1)),
            nn.ReLU(inplace=True),
        )
        for m in model.modules():
          if isinstance(m, nn.Conv2d):
            nn.init.xavier_uniform_(m.weight.data, gain=nn.init.calculate_gain('relu'))

        return model
